@@ -1,20 +1,16 @@
 from PIL import Image
+from image import FileImage
+from spriter.image import URLImage
 import os
 
 
-class URLImage(object):
-    pass
-
-
-class FileImage(object):
+class DefaultImageDoesNotExist(Exception):
+    message = "The default image path must be exist. Not found in: "
 
     def __init__(self, path):
-        self.raw = Image.open(path)
-        self.width = self.raw.size[0]
-        self.height = self.raw.size[1]
-        self.sprite_coordinate_x = None
-        self.sprite_coordinate_y = None
-        self.class_name = os.path.basename(path).split(".")[0]
+        super(DefaultImageDoesNotExist, self).__init__(
+                  self.message)
+        self.message = self.message + path
 
 
 class Sprite(object):
@@ -22,36 +18,59 @@ class Sprite(object):
     __CSS_TEMPLATE = ".{CLASSES}{{background:url(\"{ROOT_PATH}{SPRITE_NAME}\") 0 0 no-repeat}}"
     __CSS_CLASS_TEMPLATE = ".{CLASSES}.{CLASS_NAME}{{background-position: {POSITION_X}px {POSITION_Y}px}}"
 
-    def __init__(self, paths, sprite_path=None,
+    def __init__(self,
+                 paths=[],
+                 urls_paths=[],
+                 sprite_path=None,
                  sprite_name=None,
-                 sprite_url=None, image_format="RGBA", css_path="",
-                 class_name="sprite", css_name="sprite.css"):
+                 sprite_url=None,
+                 image_format="RGBA",
+                 css_path="",
+                 class_name="sprite",
+                 css_name="sprite.css",
+                 optimize=False,
+                 default_path="",
+                 default_url=""):
 
         self.images = []
         self.height = 0
         self.width = 0
+
+        if default_path is not None and default_path is not "":
+            if not isinstance(default_path, str):
+                raise TypeError(
+                            "The default image path must be a string instance")
+            if not os.path.exists(default_path):
+                raise DefaultImageDoesNotExist(default_path)
+
         for path in paths:
-            self.images.append(FileImage(path))
-        if  sprite_path:
-            self.sprite_path = sprite_path
-        else:
-            self.sprite_path = os.getcwd()
-        if sprite_url:
-            self.sprite_url = sprite_url
-        else:
-            self.sprite_url = self.sprite_path
+            img = FileImage(path, default_path)
+            self.images.append(img)
+
+        for path in urls_paths:
+            img = URLImage(path, default_url)
+            self.images.append(img)
+
         if sprite_name:
             self.sprite_name = sprite_name
         else:
             self.sprite_name = "sprite.png"
 
+        self.optimize = optimize
+
         self.image_format = image_format
         self.css_name = css_name
         self.class_name = class_name
-        self.css_path = css_path
-        self._set_sprite_image_demision()
+        self.sprite_path = sprite_path or os.getcwd() + "/"
+        self.css_path = css_path or os.getcwd() + "/"
+        self._set_sprite_image_dimension()
 
-    def _set_sprite_image_demision(self):
+        if sprite_url:
+            self.sprite_url = sprite_url
+        else:
+            self.sprite_url = self.sprite_path
+
+    def _set_sprite_image_dimension(self):
 
         for image in self.images:
 
@@ -81,38 +100,43 @@ class Sprite(object):
         css = "".join(css_line)
         return css
 
-    def do_write_css(self, css_name=None):
+    def do_write_css(self):
         """Write css's file"""
-        if not css_name:
-            css_name = self.css_name
+        if not os.path.exists(self.css_path):
+            os.makedirs(self.css_path)
 
-        path = os.path.abspath(os.curdir) + "/" + self.css_path + css_name
+        path = os.path.join(self.css_path, self.css_name)
 
         css_f = open(path, "w")
         css = self.get_css()
         css_f.write(css)
         css_f.close()
+        return path
 
     def gen_image(self):
 
-        self.image = Image.new("RGBA", (self.width, self.height))
+        self.image = Image.new(self.image_format, (self.width, self.height))
         width = 0
 
         for image in self.images:
-
             self.image.paste(image.raw,
                              (width, 0))
             width += image.width
 
     def do_write_image(self):
+        """Write sprite file"""
         if not hasattr(self, "image"):
             self.gen_image()
         if not os.path.exists(self.sprite_path):
             os.makedirs(self.sprite_path)
         path = os.path.join(self.sprite_path, self.sprite_name)
-        self.image.save(path, "PNG", options='optimize')
+        if self.optimize:
+            self.image.save(path, "PNG", optimize=1)
+        else:
+            self.image.save(path, "PNG")
         return path
 
     def gen_sprite(self):
-        self.do_write_css()
-        self.do_write_image()
+        css_path = self.do_write_css()
+        image_path = self.do_write_image()
+        return (css_path, image_path)
